@@ -634,23 +634,30 @@ panel_properties_dialog_response (PanelPropertiesDialog *dialog,
 				  int                    response,
 				  GtkWidget             *properties_dialog)
 {
-	switch (response) {
-	case GTK_RESPONSE_CLOSE:
-		gtk_widget_destroy (properties_dialog);
-		break;
-	case GTK_RESPONSE_HELP:
-		panel_show_help (gtk_window_get_screen (GTK_WINDOW (properties_dialog)),
-				 "wgospanel.xml", "gospanel-28");
-		break;
-	default:
-		break;
-	}
+        char *help_id;
+
+        switch (response) {
+        case GTK_RESPONSE_CLOSE:
+                gtk_widget_destroy (properties_dialog);
+                break;
+        case GTK_RESPONSE_HELP:
+                if (panel_toplevel_get_is_attached (dialog->toplevel)) {
+                        help_id = "gospanel-550";
+                } else {
+                        help_id = "gospanel-28";
+                }
+                panel_show_help (gtk_window_get_screen (GTK_WINDOW (properties_dialog)),
+                                 "user-guide.xml", help_id);
+                break;
+        default:
+                break;
+        }
 }
 
 static void
 panel_properties_dialog_destroy (PanelPropertiesDialog *dialog)
 {
-	panel_toplevel_unblock_auto_hide (PANEL_TOPLEVEL (dialog->toplevel));
+	panel_toplevel_pop_autohide_disabler (PANEL_TOPLEVEL (dialog->toplevel));
 	g_object_set_qdata (G_OBJECT (dialog->toplevel),
 			    panel_properties_dialog_quark,
 			    NULL);
@@ -982,7 +989,7 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 	panel_properties_dialog_update_for_attached (dialog,
 						     panel_toplevel_get_is_attached (dialog->toplevel));
 
-	panel_toplevel_block_auto_hide (dialog->toplevel);
+	panel_toplevel_push_autohide_disabler (dialog->toplevel);
 
 	gtk_widget_show (dialog->properties_dialog);
 
@@ -994,7 +1001,6 @@ panel_properties_dialog_present (PanelToplevel *toplevel)
 {
 	PanelPropertiesDialog *dialog;
 	GladeXML              *gui;
-	char                  *glade_file;
 
 	if (!panel_properties_dialog_quark)
 		panel_properties_dialog_quark =
@@ -1008,12 +1014,9 @@ panel_properties_dialog_present (PanelToplevel *toplevel)
 		return;
 	}
 
-	if (g_file_test ("panel-properties-dialog.glade", G_FILE_TEST_EXISTS))
-		glade_file = "panel-properties-dialog.glade";
-	else
-		glade_file = GLADEDIR "/panel-properties-dialog.glade";
-
-	gui = glade_xml_new (glade_file, "panel_properties_dialog", NULL);
+	gui = glade_xml_new (GLADEDIR "/panel-properties-dialog.glade",
+			     "panel_properties_dialog",
+			     NULL);
 
 	dialog = panel_properties_dialog_new (toplevel, gui);
 
